; Aline
; by Hikaru/Intense in 2016
; SjASMPlus syntax
; Public Domain


; FBDetect
;
; One-time floating bus port detection.
; Sets up aline to use the detected port number.
; Expects interrupts to be enabled and end in under ~14000 T states.
; Port numbers are #FFFF for the Sinclair, #0FFD for the Amstrad models.
;
; Returns CF=0 if found, BC = port number,
;	  CF=1 if not found.
FBDetect
	ld hl,#5AFF
	ld (hl),l
	ld c,l
	ld b,l
	xor a
	dec hl
	ld (hl),a
	cp (hl)
	jr z,$-3
	halt
	halt
	call .scan
	ret nc
	ld bc,#0FFD
.scan
	ld (Aline.LOC_Port),bc
	xor a
	ld h,a
.scan_lp
	in l,(c)
	ret p
	dec a
	jr nz,.scan_lp
	dec h
	jr nz,.scan_lp
	scf
	ret


; SyncBorder
;
; Delays program execution until the lower border.
; Should be called before a call to Aline.
; Unnecessary if the main loop of your program
; is guaranteed to always finish at the lower border.
SyncBorder
	ld hl,#4000
	ld (hl),#FF
	ld d,4
	ld bc,(Aline.LOC_Port)
.lp1
	ld e,d
.lp2
	ld a,(hl)
	in a,(c)
	inc a
	jr nz,.lp1
	dec e
	jr nz,.lp2
	ret


; Aline
;
; Delays execution until a certain T state in the frame.
; The exact value depends on the model:
; ZX Spectrum 48: 14524
; ZX Spectrum 128, +2: 14550
; ZX Spectrum +2A, +3: 14552~14556 (5T precision)
; Should be run during border time.
Aline
	ld hl,#4000
	ld bc,#5841
	ld de,#FF02
	ld a,#7A
.fill_lp
	ld (hl),a
	ld h,b
	ld (hl),0
	inc l
	sub e
	ld (hl),a
	ld h,c
	ld (hl),d	;not actually needed in LSB=#1E
	dec h
	ld (hl),d	;#FF would be returned over the border on the +2A/+3
	inc l
	sub 6
	jr nc,.fill_lp
	ld d,16
	call .sync
	sub d		;Delay 16T for (A / 16)
	jr nc,$-1
	rra		;Bit 0 ignored
	rra
	jr nc,$+4	;Bit 1: pixels, delay 3T to equalize with attrs
	nop
	nop
	and e		;Bit 2 ignored
	ret z		;Bit 3: delay 8T
	xor a
	ret
.sync
	ld hl,.sync_lp
.LOC_Port=$+1
	ld bc,#FFFF
.sync_lp
	in a,(c)	;12
	cp e		;4
	ret p		;5
	jp (hl)		;4 25